use std::sync::Arc;

use crate::{
    commands::command_base::{CommandDeserializer, DeserializedCommand},
    state::{audio::AtemClassicAudioState, info::AudioMixerInfo},
};

pub const DESERIALIZE_AUDIO_MIXER_CONFIG_NAME: &str = "_AMC";

#[derive(Debug)]
pub struct AudioMixerConfig {
    inputs: u8,
    monitors: u8,
    headphones: u8,
}

impl DeserializedCommand for AudioMixerConfig {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_AUDIO_MIXER_CONFIG_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.audio_mixer = Some(AudioMixerInfo::new(
            self.inputs,
            self.monitors,
            self.headphones,
        ));
        state.audio = Some(AtemClassicAudioState::new(self.inputs, self.monitors != 0))
    }
}

#[derive(Default)]
pub struct AudioMixerConfigDeserializer {}

impl CommandDeserializer for AudioMixerConfigDeserializer {
    fn deserialize(
        &self,
        buffer: &[u8],
        version: &crate::enums::ProtocolVersion,
    ) -> std::sync::Arc<dyn DeserializedCommand> {
        Arc::new(AudioMixerConfig {
            inputs: buffer[0],
            monitors: buffer[1],
            headphones: buffer[2],
        })
    }
}
