use std::sync::Arc;

use crate::{
    commands::command_base::{CommandDeserializer, DeserializedCommand},
    state::info::MediaPoolInfo,
};

pub const DESERIALIZE_MEDIA_POOL_CONFIG_NAME: &str = "_mpl";

#[derive(Debug)]
pub struct MediaPoolConfig {
    still_count: u8,
    clip_count: u8,
}

impl DeserializedCommand for MediaPoolConfig {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_MEDIA_POOL_CONFIG_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.media_pool = Some(MediaPoolInfo::new(self.still_count, self.clip_count))
    }
}

#[derive(Default)]
pub struct MediaPoolConfigDeserializer {}

impl CommandDeserializer for MediaPoolConfigDeserializer {
    fn deserialize(
        &self,
        buffer: &[u8],
        _version: &crate::enums::ProtocolVersion,
    ) -> std::sync::Arc<dyn DeserializedCommand> {
        Arc::new(MediaPoolConfig {
            still_count: buffer[0],
            clip_count: buffer[1],
        })
    }
}
