use std::sync::Arc;

use crate::{
    commands::command_base::{CommandDeserializer, DeserializedCommand},
    state::info::MixEffectInfo,
};

pub const DESERIALIZE_MIX_EFFECT_BLOCK_CONFIG_NAME: &str = "_MeC";

#[derive(Debug)]
pub struct MixEffectBlockConfig {
    mix_effect: u8,
    key_count: u8,
}

impl DeserializedCommand for MixEffectBlockConfig {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_MIX_EFFECT_BLOCK_CONFIG_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.mix_effects[self.mix_effect as usize] = Some(MixEffectInfo::new(self.key_count));
    }
}

#[derive(Default)]
pub struct MixEffectBlockConfigDeserializer {}

impl CommandDeserializer for MixEffectBlockConfigDeserializer {
    fn deserialize(
        &self,
        buffer: &[u8],
        _version: &crate::enums::ProtocolVersion,
    ) -> std::sync::Arc<dyn DeserializedCommand> {
        Arc::new(MixEffectBlockConfig {
            mix_effect: buffer[0],
            key_count: buffer[1],
        })
    }
}
