use std::sync::Arc;

use crate::{
    commands::command_base::{CommandDeserializer, DeserializedCommand},
    enums::ProtocolVersion,
    state::info::MultiviewerInfo,
};

pub const DESERIALIZE_MULTIVIEWER_NAME: &str = "_MvC";

#[derive(Debug)]
pub struct MultiviewerConfig {
    count: Option<u8>,
    window_count: u8,
}

impl DeserializedCommand for MultiviewerConfig {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_MULTIVIEWER_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        // TODO: This can't be right...

        let existing_count = match &state.info.multiviewer {
            Some(multiviewer) => multiviewer.count().as_ref().copied(),
            None => None,
        };
        let count = match self.count {
            Some(count) => Some(count),
            None => existing_count,
        };
        state.info.multiviewer = Some(MultiviewerInfo::new(count, self.window_count));
    }
}

#[derive(Default)]
pub struct MultiviewerConfigDeserializer {}

impl CommandDeserializer for MultiviewerConfigDeserializer {
    fn deserialize(
        &self,
        buffer: &[u8],
        version: &crate::enums::ProtocolVersion,
    ) -> std::sync::Arc<dyn DeserializedCommand> {
        if *version >= ProtocolVersion::V8_1_1 {
            Arc::new(MultiviewerConfig {
                count: None,
                window_count: buffer[1],
            })
        } else {
            Arc::new(MultiviewerConfig {
                count: Some(buffer[0]),
                window_count: buffer[1],
            })
        }
    }
}
