use crate::{commands::command_base::DeserializedCommand, enums::ProtocolVersion};

pub const DESERIALIZE_VERSION_RAW_NAME: &str = "_ver";

#[derive(Debug)]
pub struct Version {
    pub version: ProtocolVersion,
}

impl DeserializedCommand for Version {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_VERSION_RAW_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.api_version = self.version.clone();
    }
}

pub fn deserialize_version(buffer: &[u8]) -> Version {
    let version = u32::from_be_bytes([buffer[0], buffer[1], buffer[2], buffer[3]]);
    let version: ProtocolVersion = version.try_into().expect("Invalid protocol version");

    Version { version }
}
