use std::sync::Arc;

use crate::enums::ProtocolVersion;

use super::command_base::{CommandDeserializer, DeserializedCommand};

pub const DESERIALIZE_TIME_RAW_NAME: &str = "Time";

#[derive(Debug)]
pub struct TimeInfo {
    pub hour: u8,
    pub minute: u8,
    pub second: u8,
    pub frame: u8,
    pub drop_frame: bool,
}

#[derive(Debug)]
pub struct Time {
    info: TimeInfo,
}

impl DeserializedCommand for Time {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_TIME_RAW_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {}
}

#[derive(Default)]
pub struct TimeDeserializer {}

impl CommandDeserializer for TimeDeserializer {
    fn deserialize(
        &self,
        buffer: &[u8],
        version: &ProtocolVersion,
    ) -> Arc<dyn DeserializedCommand> {
        let info = TimeInfo {
            hour: buffer[0],
            minute: buffer[1],
            second: buffer[2],
            frame: buffer[3],
            drop_frame: buffer[5] == 1,
        };

        Arc::new(Time { info })
    }
}
