use crate::enums::{Model, ProtocolVersion};

#[derive(Clone, PartialEq, Getters, new)]
pub struct AtemCapabilites {
    mix_effects: u8,
    sources: u8,
    auxilliaries: u8,
    mix_minus_outputs: u8,
    media_players: u8,
    serial_ports: u8,
    max_hyperdecks: u8,
    dves: u8,
    stingers: u8,
    super_sources: u8,
    talkback_channels: u8,
    downstream_keyers: u8,
    camera_control: bool,
    advanced_chroma_keyers: bool,
    only_configurable_outputs: bool,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MixEffectInfo {
    key_count: u8,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct SuperSourceInfo {
    box_count: u64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct AudioMixerInfo {
    inputs: u8,
    monitors: u8,
    headphones: u8,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct FairlightAudioMixerInfo {
    inputs: u64,
    monitors: u64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MacroPoolInfo {
    macro_count: u64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MediaPoolInfo {
    still_count: u8,
    clip_count: u8,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MultiviewerInfo {
    count: Option<u8>,
    window_count: u8,
}

#[derive(Clone, PartialEq, new)]
pub struct TimeInfo {
    pub hour: u64,
    pub minute: u64,
    pub second: u64,
    pub frame: u64,
    pub drop_frame: bool,
}

#[derive(Clone, PartialEq, Getters, new, Default)]
pub struct DeviceInfo {
    pub api_version: ProtocolVersion,
    pub capabilities: Option<AtemCapabilites>,
    pub model: Model,
    pub product_identifier: Option<String>,
    pub super_sources: Vec<Option<SuperSourceInfo>>,
    pub mix_effects: Vec<Option<MixEffectInfo>>,
    pub power: Vec<bool>,
    pub audio_mixer: Option<AudioMixerInfo>,
    pub fairlight_mixer: Option<FairlightAudioMixerInfo>,
    pub macro_pool: Option<MacroPoolInfo>,
    pub media_pool: Option<MediaPoolInfo>,
    pub multiviewer: Option<MultiviewerInfo>,
    pub last_time: Option<TimeInfo>,
}
