use crate::enums;

#[derive(Clone, PartialEq, Getters, new)]
pub struct MediaPlayer {
    pub playing: bool,
    pub is_loop: bool,
    pub at_beginning: bool,
    pub clip_frame: u64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MediaPlayerSource {
    pub source_type: enums::MediaSourceType,
    pub clip_index: u64,
    pub still_index: u64,
}

pub type MediaPlayerState = (MediaPlayer, MediaPlayerSource);

#[derive(Clone, PartialEq, Getters, new, Default)]
pub struct MediaState {
    still_pool: Vec<StillFrame>,
    clip_pool: Vec<ClipBank>,
    players: Vec<MediaPlayerState>,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct StillFrame {
    pub is_used: bool,
    pub hash: String,
    pub file_name: String,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct ClipBank {
    pub is_used: bool,
    pub name: String,
    pub frame_count: u64,
    pub frames: Vec<StillFrame>,
}
