use std::collections::HashMap;

pub mod atem_macro;
pub mod audio;
pub mod color;
pub mod common;
pub mod fairlight;
pub mod info;
pub mod input;
pub mod media;
pub mod recording;
pub mod settings;
pub mod streaming;
pub mod util;
pub mod video;

#[derive(Default, Clone, PartialEq)]
pub struct AtemState {
    pub info: info::DeviceInfo,
    pub video: video::AtemVideoState,
    pub audio: Option<audio::AtemClassicAudioState>,
    pub fairlight: Option<fairlight::AtemFairlightAudioState>,
    pub media: media::MediaState,
    pub inputs: HashMap<u64, input::InputChannel>,
    // macro is a rust keyword
    pub atem_macro: atem_macro::MacroState,
    pub settings: settings::SettingsState,
    pub recording: Option<recording::RecordingState>,
    pub streaming: Option<streaming::StreamingState>,
    pub color_generators: HashMap<u64, color::ColorGeneratorState>,
}
