pub trait DownstreamKeyerBase {
    fn get_in_transition(&self) -> bool;
    fn get_remaining_frames(&self) -> f64;
    fn get_is_auto(&self) -> bool;

    fn get_on_air(&self) -> bool;
    fn set_on_air(&mut self, on_air: bool);
    fn get_is_towards_on_air(&self) -> Option<bool>;
    fn set_is_towards_on_air(&mut self, on_air: Option<bool>);
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct DownstreamKeyer {
    pub sources: Option<DownstreamKeyerSources>,
    pub properties: Option<DownstreamKeyerProperties>,

    in_transition: bool,
    remaining_frames: f64,
    is_auto: bool,
    pub on_air: bool,
    pub is_towards_air: Option<bool>,
}

impl DownstreamKeyerBase for DownstreamKeyer {
    fn get_in_transition(&self) -> bool {
        self.in_transition
    }
    fn get_remaining_frames(&self) -> f64 {
        self.remaining_frames
    }
    fn get_is_auto(&self) -> bool {
        self.is_auto
    }

    fn get_on_air(&self) -> bool {
        self.on_air
    }
    fn set_on_air(&mut self, on_air: bool) {
        self.on_air = on_air
    }
    fn get_is_towards_on_air(&self) -> Option<bool> {
        self.is_towards_air
    }
    fn set_is_towards_on_air(&mut self, on_air: Option<bool>) {
        self.is_towards_air = on_air
    }
}

pub trait DownstreamKeyerGeneral {
    fn get_pre_multiply(&self) -> bool;
    fn set_pre_multiply(&mut self, pre_multiply: bool);
    fn get_clip(&self) -> f64;
    fn set_clip(&mut self, clip: f64);
    fn get_gain(&self) -> f64;
    fn set_gain(&mut self, gain: f64);
    fn get_invert(&self) -> bool;
    fn set_invert(&mut self, invert: bool);
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct DownstreamKeyerMask {
    pub enabled: bool,
    pub top: f64,
    pub bottom: f64,
    pub left: f64,
    pub right: f64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct DownstreamKeyerProperties {
    pub tie: bool,
    pub rate: f64,
    pub mask: DownstreamKeyerMask,
    pub pre_multiply: bool,
    pub clip: f64,
    pub gain: f64,
    pub invert: bool,
}

impl DownstreamKeyerGeneral for DownstreamKeyerProperties {
    fn get_pre_multiply(&self) -> bool {
        self.pre_multiply
    }
    fn set_pre_multiply(&mut self, pre_multiply: bool) {
        self.pre_multiply = pre_multiply
    }
    fn get_clip(&self) -> f64 {
        self.clip
    }
    fn set_clip(&mut self, clip: f64) {
        self.clip = clip
    }
    fn get_gain(&self) -> f64 {
        self.gain
    }
    fn set_gain(&mut self, gain: f64) {
        self.gain = gain
    }
    fn get_invert(&self) -> bool {
        self.invert
    }
    fn set_invert(&mut self, invert: bool) {
        self.invert = invert
    }
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct DownstreamKeyerSources {
    pub fill_source: f64,
    pub cut_source: f64,
}
