use crate::enums;

mod downstream_keyers;
mod super_source;
mod upstream_keyers;

#[derive(Clone, PartialEq, Getters, new)]
pub struct DipTransitionSettings {
    pub rate: f64,
    pub input: f64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct DVETransitionSettings {
    pub rate: f64,
    pub logo_rate: f64,
    pub style: enums::DVEEffect,
    pub fill_source: f64,
    pub key_source: f64,

    pub enable_key: f64,
    pub pre_multiplied: bool,
    pub clip: f64,
    pub gain: f64,
    pub invert_key: bool,
    pub reverse: bool,
    pub flip_flop: bool,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MixTransitionSettings {
    pub rate: f64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct StingerTransitionSettings {
    pub source: f64,
    pub pre_multiplied_key: bool,

    pub clip: f64,
    pub gain: f64,
    pub invert: bool,

    pub preroll: f64,
    pub clip_duration: f64,
    pub trigger_point: f64,
    pub mix_rate: f64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct WipeTransitionSettings {
    pub rate: f64,
    pub pattern: f64,
    pub border_width: f64,
    pub border_input: f64,
    pub symmetry: f64,
    pub border_softenss: f64,
    pub x_position: f64,
    pub y_position: f64,
    pub reverse_direction: bool,
    pub flip_flop: bool,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct TransitionProperties {
    style: enums::TransitionStyle,
    selection: Vec<enums::TransitionSelection>,
    pub next_style: enums::TransitionStyle,
    pub next_selection: Vec<enums::TransitionSelection>,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct TransitionSettings {
    pub dip: Option<DipTransitionSettings>,
    pub dve: Option<DVETransitionSettings>,
    pub mix: Option<MixTransitionSettings>,
    pub stinger: Option<StingerTransitionSettings>,
    pub wipe: Option<WipeTransitionSettings>,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct TransitionPosition {
    in_transition: bool,
    remaining_frames: f64,
    pub handle_position: f64,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct MixEffect {
    index: usize,
    pub program_input: u16,
    pub preview_input: u16,
    pub transition_preview: bool,
    pub fade_to_black: Option<FadeToBlackProperties>,
    pub transition_position: TransitionPosition,
    pub transition_properties: TransitionProperties,
    pub transition_settings: TransitionSettings,
    upstream_keyers: Vec<upstream_keyers::UpstreamKeyer>,
}

#[derive(Clone, PartialEq, Getters, new)]
pub struct FadeToBlackProperties {
    is_fully_black: bool,
    in_transition: bool,
    remaining_frames: f64,
    pub rate: f64,
}

#[derive(Clone, PartialEq, Getters, new, Default)]
pub struct AtemVideoState {
    mix_effects: Vec<MixEffect>,
    downstream_keyers: Vec<downstream_keyers::DownstreamKeyer>,
    auxiliaries: Vec<f64>,
    super_sources: Vec<super_source::SuperSource>,
}

impl AtemVideoState {
    pub fn mix_effects_mut(&mut self) -> &mut Vec<MixEffect> {
        &mut self.mix_effects
    }
}
