use crate::enums;

pub trait UpstreamKeyerBase: UpstreamKeyerTypeSettings {
    fn get_upstream_keyer_id(&self) -> f64;
    fn get_can_fly_key(&self) -> bool;
    fn get_fill_source(&self) -> f64;
    fn set_fill_source(&mut self, source: f64);
    fn get_cut_source(&self) -> f64;
    fn set_cut_source(&mut self, source: f64);
}

pub trait UpstreamKeyerTypeSettings {
    fn get_mix_effect_key_type(&self) -> enums::MixEffectKeyType;
    fn set_mix_effect_key_type(&mut self, key_type: enums::MixEffectKeyType);
    fn get_fly_enabled(&self) -> bool;
    fn set_fly_enabled(&mut self, enabled: bool);
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerDVESettings {
    pub border_enabled: bool,
    pub shadow_enabled: bool,
    pub border_bevel: enums::BorderBevel,
    pub rate: f64,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerFlyKeyFrame {
    key_frame_id: f64,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

#[derive(Clone, PartialEq)]
pub enum UpstreamKeyerMaskSettings {
    DVE(UpstreamKeyerDVESettings),
    FlyKeyFrame(UpstreamKeyerFlyKeyFrame),
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerChromaSettings {
    pub hue: f64,
    pub gain: f64,
    pub y_suppress: f64,
    pub lift: f64,
    pub narrow: bool,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerAdvancedChromaSettings {
    pub properties: Option<UpstreamKeyerAdvancedChromaProperties>,
    pub sample: Option<UpstreamKeyerAdvancedChromaSample>,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerAdvancedChromaProperties {
    pub foreground_level: f64,
    pub background_level: f64,
    pub key_edge: f64,

    pub spill_suppression: f64,
    pub flare_suppression: f64,

    pub brightness: f64,
    pub contrast: f64,
    pub saturation: f64,
    pub red: f64,
    pub green: f64,
    pub blue: f64,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerAdvancedChromaSample {
    pub enable_cursor: bool,
    pub preview: bool,
    pub cursor_x: f64,
    pub cursor_y: f64,
    pub cursor_size: f64,
    pub sampled_y: f64,
    pub sampled_cb: f64,
    pub sampled_cr: f64,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerLumaSettings {
    pub pre_multiplied: bool,
    pub clip: f64,
    pub gain: f64,
    pub invert: bool,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerPatternSettings {
    pub style: enums::Pattern,
    pub size: f64,
    pub symmetry: f64,
    pub softness: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub invert: bool,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyerFlySettings {
    is_a_set: bool,
    is_b_set: bool,
    is_at_key_frame: enums::IsAtKeyFrame,
    run_to_infinite_index: f64,
}

#[derive(Clone, PartialEq)]
pub struct UpstreamKeyer {
    pub mix_effect_key_type: enums::MixEffectKeyType,
    pub fly_enabled: bool,

    upstream_keyer_id: f64,
    can_fly_key: bool,
    pub fill_source: f64,
    pub cut_source: f64,

    pub dve_settings: Option<UpstreamKeyerDVESettings>,
    pub chroma_settings: Option<UpstreamKeyerChromaSettings>,
    pub advanced_chroma_settings: Option<UpstreamKeyerAdvancedChromaSettings>,
    pub luma_settings: Option<UpstreamKeyerLumaSettings>,
    pub pattern_settings: Option<UpstreamKeyerPatternSettings>,
    pub fly_keyframes: [Option<UpstreamKeyerFlyKeyFrame>; 2],
    pub fly_properties: Option<UpstreamKeyerFlySettings>,
    pub mask_settings: UpstreamKeyerMaskSettings,
    pub on_air: bool,

    pub mask_enabled: bool,
    pub mask_top: f64,
    pub mask_bottom: f64,
    pub mask_left: f64,
    pub mask_right: f64,

    pub size_x: f64,
    pub size_y: f64,
    pub position_x: f64,
    pub position_y: f64,
    pub rotation: f64,

    pub border_outer_width: f64,
    pub border_inner_width: f64,
    pub border_outer_softness: f64,
    pub border_inner_softness: f64,
    pub border_bevel_softness: f64,
    pub border_bevel_position: f64,

    pub border_opacity: f64,
    pub border_hue: f64,
    pub border_saturation: f64,
    pub border_luma: f64,

    pub light_source_direction: f64,
    pub light_source_altitude: f64,
}

impl UpstreamKeyerTypeSettings for UpstreamKeyer {
    fn get_mix_effect_key_type(&self) -> enums::MixEffectKeyType {
        self.mix_effect_key_type
    }

    fn set_mix_effect_key_type(&mut self, key_type: enums::MixEffectKeyType) {
        self.mix_effect_key_type = key_type
    }

    fn get_fly_enabled(&self) -> bool {
        self.fly_enabled
    }

    fn set_fly_enabled(&mut self, enabled: bool) {
        self.fly_enabled = enabled
    }
}

impl UpstreamKeyerBase for UpstreamKeyer {
    fn get_upstream_keyer_id(&self) -> f64 {
        self.upstream_keyer_id
    }

    fn get_can_fly_key(&self) -> bool {
        self.can_fly_key
    }

    fn get_fill_source(&self) -> f64 {
        self.fill_source
    }

    fn set_fill_source(&mut self, source: f64) {
        self.fill_source = source
    }

    fn get_cut_source(&self) -> f64 {
        self.cut_source
    }

    fn set_cut_source(&mut self, source: f64) {
        self.cut_source = source
    }
}
